function [fhat,time,numiter]=multi_lattice_ifft_noniter_avg(f,h,Ms,zs,err,opts)
%MULTI_LATTICE_IFFT_NONITER_AVG Uses multiple rank-1 lattice FFTs to reconstruct all Fourier coefficients.

    if (nargin<5) || isempty(err)
        err=1e-10;
    end

    if(nargin<6 || ~isfield(opts,'alg'))
        opts.alg='direct';
    end

    switch(opts.alg)
        case 'cg'
            [fhat,time,numiter]=ml_ifft_cg(f,h,Ms,zs,err,opts);
        case 'direct'
            [fhat, time]=ml_ifft_direct(f,h,Ms,zs);
            numiter=0;
        case 'consecutive'
            [fhat, time1]=ml_ifft_direct(f,h,Ms,zs);
            opts.start=fhat;
            [fhat,time2,numiter]=ml_ifft_cg(f,h,Ms,zs,err,opts);
            time=time1+time2;
    end

end

function [fhat,time,numiter]=ml_ifft_cg(f,h,Ms,zs,err,opts)

tic;
f1=multi_lattice_fft(f,h,Ms,zs,'transp');
% tic;
% fhat=gmres(@(x) multi_lattice_fft(x,h,Ms,zs,'both'),f1,10,err,1000);
% time(1)=toc;
if(nargin<6 || ~isfield(opts, 'start'))
    opts.start=zeros(size(h,1),1);
end

[fhat,numiter]=multi_lattice_dirtycg(@(x) multi_lattice_fft(x,h,Ms,zs,'both'),f1,err,opts);
%[fhat,tmp1,tmp2,numiter]=pcg(@(x) multi_lattice_fft(x,h,Ms,zs,'both'),f1,err,10000,[],[],opts.start);
% pcg und lsqr haben sich hier nicht bewährt!!!
time=toc;

%fhat=[fhat,fhat1];

end

function [fhat, time]=ml_ifft_direct(f,h,Ms,zs)
    tic;

    fhat = zeros(size(h,1),1);
    counter = zeros(size(h,1),1);

    for j=1:length(Ms)
      y = mod(h*zs(j,:)',Ms(j)) + 1;
      num_1d = accumarray(y,1,[Ms(j),1],@sum);
      num_alias = num_1d(y);
      ind_reco = find(num_alias == 1);
      
%       [y1,ind1]=sort(y);
%       ind2 = find([y1(2:end)-y1(1:end-1); y1(end)-y1(end-1)]~=0);
%       ind_reco = ind1(ind2);
% 
      if j==1
        f1=f(1:Ms(1));
      else
        f1=[f(1);f(sum(Ms(1:j-1))-j+3:sum(Ms(1:j))-j+1)];
      end

      a1 = 1/Ms(j)*lfft(h(ind_reco,:),zs(j,:),Ms(j),f1,'transp');
%       vv1 = y(ind_reco);
%       vv2 = y1(ind2);
      fhat(ind_reco) = fhat(ind_reco) + a1;
      counter(ind_reco) = counter(ind_reco) + 1;
    end

    fhat = fhat ./ counter;

    time = toc;
end

