function [results] = test_mr1l_approx_rand_fulllattice1(N,d,s_array,nrepeat)
%TEST_MR1L_APPROX_RAND_FULLLATICE1 Computes the number of samples in multiple rank-1 lattices constructed by Algorithm 2.1 for random frequency sets crosses starting with exponential lattices for the full grid.

filename = sprintf('test_mr1l_approx_rand_fulllattice1_N%d_d%d.mat',N, d);

rng_state = rng;

results = {};

if max(s_array) > (2*N+1)^d
  error('max s > full cube');
end
  
for iis = 1:length(s_array)
 s = s_array(iis);
 for ir = 1:nrepeat
  result = struct();
  result.d = d;
  result.N = N;
  result.num_freq = s;
  
  freq = unique(randi(2*N+1, 2*s, d) - (N+1), 'rows', 'stable');
  while size(freq,1) < s
    freq = unique([freq; randi(2*N+1, 2*s, d) - (N+1)], 'rows', 'stable');
  end
  freq = freq(1:s,:);
  
  N_I = max(max(freq,[],1)-min(freq,[],1)+1);
  M = N_I^d;
  fprintf('N=%d, d=%d, s=%d, M=%g, ', N, d, result.num_freq, M);

  if (N_I^(d-1) > 2^62) || (max(sum(abs(freq),2))*N_I^(d-1) > 2^62) || (N_I^(d-1) > flintmax)
    z = cell(1,d);
    z{1} = int64(1);
    for t=2:d
%       z_t = int64(repmat(N_I,1,t-1));
      z_t = int64([]);
      max_group = max( 1 , min(floor(log(2^62)/log(N_I)-1),floor(log(flintmax)/log(N_I))) );
      num_groups = floor((t-1) / max_group);
      for ii=1:num_groups
        z_t = [z_t, int64(N_I^max_group)];
      end
      if t-1 > max_group * num_groups
        z_t = [z_t, int64(N_I^(t-1-max_group*num_groups))];
      end
      z{t} = z_t;
    end
    fprintf('z large, ');
  else
    z = N_I.^(0:d-1);
  end

  result.z = z;
  result.M = M;
  [Ms,hred_array,hrem_array] = deterministic_mr1l_approx_get_lattice_sizes(freq,z,M);
  fprintf('numlat=%d, samples=%d\n', length(Ms), sum(Ms)-length(Ms)+1);
  result.Ms = Ms;
  result.hred_array = hred_array;
  result.hrem_array = hrem_array;
%[[Ms;0], [size(freq,1);hrem_array]];
  results{iis,ir} = result;
 end
 save(filename', 'N', 'd', 's_array', 'results', 'rng_state', '-v7.3');
end

